@@ -134,9 +134,9 @@ seahorse_context_init (SeahorseContext *sctx)
 }
 
 static void
-hash_to_slist (gpointer object, gpointer value, gpointer user_data)
+hash_to_ref_slist (gpointer object, gpointer value, gpointer user_data)
 {
-	*((GSList**)user_data) = g_slist_prepend (*((GSList**)user_data), value);
+	*((GSList**)user_data) = g_slist_prepend (*((GSList**)user_data), g_object_ref (value));
 }
 
 /* release all references */
@@ -150,9 +150,11 @@ seahorse_context_dispose (GObject *gobject)
     
     /* Release all the objects */
     objects = NULL;
-    g_hash_table_foreach (sctx->pv->objects_by_source, hash_to_slist, &objects);
-    for (l = objects; l; l = g_slist_next (l)) 
-	    seahorse_context_remove_object (sctx, l->data);
+    g_hash_table_foreach (sctx->pv->objects_by_source, hash_to_ref_slist, &objects);
+    for (l = objects; l; l = g_slist_next (l)) {
+            seahorse_context_remove_object (sctx, l->data);
+            g_object_unref (l->data);
+    }
     g_slist_free (objects);
 
     /* Gconf notification */
@@ -705,6 +707,26 @@ seahorse_context_find_objects (SeahorseContext *sctx, GQuark ktype,
     return seahorse_context_find_objects_full (sctx, &pred);
 }
 
+static void
+verify_each_object (gpointer key, gpointer value, gpointer user_data)
+{
+	gpointer k;
+
+	g_assert (SEAHORSE_IS_OBJECT (value));
+	k = hashkey_by_source (seahorse_object_get_source (value), 
+	                       seahorse_object_get_id (value));
+	g_assert (k == key);
+}
+
+void
+seahorse_context_verify_objects (SeahorseContext *self)
+{
+	if (!self)
+		self = seahorse_context_for_app ();
+	g_return_if_fail (SEAHORSE_IS_CONTEXT (self));
+	g_hash_table_foreach (self->pv->objects_by_source, verify_each_object, self);
+}
+
 void 
 seahorse_context_remove_object (SeahorseContext *sctx, SeahorseObject *sobj)
 {
@@ -1072,6 +1094,7 @@ GList*
 seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype, 
                                    GSList *rawids)
 {
+    SeahorseOperation *op = NULL;
     GList *robjects = NULL;
     GQuark id = 0;
     GSList *todiscover = NULL;
@@ -1079,7 +1102,6 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
     SeahorseSource *sksrc;
     SeahorseObject* sobj;
     SeahorseLocation loc;
-    SeahorseOperation *op;
     GSList *l;
 
     if (!sctx)
@@ -1137,27 +1159,27 @@ seahorse_context_discover_objects (SeahorseContext *sctx, GQuark ktype,
         /* Running operations ref themselves */
         g_object_unref (op);
     }
-    
+
     /* Start a discover process on all todiscover */
     if (seahorse_gconf_get_boolean (AUTORETRIEVE_KEY) && todiscover) {
         op = seahorse_context_retrieve_objects (sctx, ktype, todiscover, NULL);
-
-        /* Add unknown objects for all these */
-        sksrc = seahorse_context_find_source (sctx, ktype, SEAHORSE_LOCATION_MISSING);
-        for (l = todiscover; l; l = g_slist_next (l)) {
-            if (sksrc) {
-                sobj = seahorse_unknown_source_add_object (SEAHORSE_UNKNOWN_SOURCE (sksrc), 
-                                                           GPOINTER_TO_UINT (l->data), op);
-                robjects = g_list_prepend (robjects, sobj);
-            }
-        }
-        
-        g_slist_free (todiscover);
         
         /* Running operations ref themselves */
         g_object_unref (op);
     }
-    
+
+    /* Add unknown objects for all these */
+    sksrc = seahorse_context_find_source (sctx, ktype, SEAHORSE_LOCATION_MISSING);
+    for (l = todiscover; l; l = g_slist_next (l)) {
+        if (sksrc) {
+            sobj = seahorse_unknown_source_add_object (SEAHORSE_UNKNOWN_SOURCE (sksrc), 
+                                                       GPOINTER_TO_UINT (l->data), op);
+            robjects = g_list_prepend (robjects, sobj);
+        }
+    }
+
+    g_slist_free (todiscover);
+
     return robjects;
 }
 
